using Toybox.WatchUi as Ui;
using Toybox.Graphics as Gfx;
using Toybox.Application as App;
using Toybox.System as Sys;
using Toybox.Time;
using Toybox.Time.Gregorian;

enum /* GOAL_TYPES */ {
	GOAL_TYPE_BATTERY = -1,
	GOAL_TYPE_CALORIES = -2,
	GOAL_TYPE_OFF = -3,

	GOAL_TYPE_STEPS = 0, // App.GOAL_TYPE_STEPS
	GOAL_TYPE_FLOORS_CLIMBED, // App.GOAL_TYPE_FLOORS_CLIMBED
	GOAL_TYPE_ACTIVE_MINUTES // App.GOAL_TYPE_ACTIVE_MINUTES
}

class DataArea extends Ui.Drawable {

	private var mRow1Y1;
	private var mRow2Y1;
	private var mRow1Y2;
	private var mRow2Y2;

	private var mFirstGoalType;
	private var mFirstGoalIsValid;
	private var mFirstGoalCurrent;
	private var mFirstGoalMax;

	private var mSecondGoalType;
	private var mSecondGoalIsValid;
	private var mSecondGoalCurrent;
	private var mSecondGoalMax;

	private var mGoalIconY1;
	private var mGoalIconY2;
	private var mGoalIconX1;
	private var mGoalIconX2;
	
	private var mX1;
	private var mX2;

	function initialize(params) {
		Drawable.initialize(params);

		
		mRow1Y1 = params[:row1Y1];
		mRow2Y1 = params[:row2Y1];
		mRow1Y2 = params[:row1Y2];
		mRow2Y2 = params[:row2Y2];

		mGoalIconY1 = params[:goalIconY1];
		mGoalIconY2 = params[:goalIconY2];
		mGoalIconX1 = params[:goalIconX1];
		mGoalIconX2 = params[:goalIconX2];
		
		mX1 = params[:locX1];
		mX2 = params[:locX2];
	}

	function setGoalValues(firstType, firstValues, secondType, secondValues) {
		mFirstGoalType = firstType;
		mFirstGoalIsValid = firstValues[:isValid];

		if (firstValues[:isValid]) {
			mFirstGoalCurrent = firstValues[:current].format(INTEGER_FORMAT);
			mFirstGoalMax =  firstValues[:max].format(INTEGER_FORMAT);
		} else {
			mFirstGoalCurrent = null;
			mFirstGoalMax = null;
		}

		mSecondGoalType = secondType;
		mSecondGoalIsValid = secondValues[:isValid];

		if (secondValues[:isValid]) {
			mSecondGoalCurrent = secondValues[:current].format(INTEGER_FORMAT);
			mSecondGoalMax =  secondValues[:max].format(INTEGER_FORMAT);
		} else {
			mSecondGoalCurrent = null;
			mSecondGoalMax = null;
		}
	} 

	function draw(dc) {
	
		if (mFirstGoalType != -3){ 
			drawGoalIcon(dc, mGoalIconX1, mFirstGoalType, mFirstGoalIsValid, Graphics.TEXT_JUSTIFY_LEFT,mGoalIconY1);
		}
		if (mSecondGoalType != -3){ 
			drawGoalIcon(dc, mGoalIconX2, mSecondGoalType, mSecondGoalIsValid, Graphics.TEXT_JUSTIFY_LEFT,mGoalIconY2);
		}
		//var city = App.getApp().getProperty("LocalTimeInCity");

		// #78 Setting with value of empty string may cause corresponding property to be null.
		/*if ((city != null) && (city.length() != 0)) {
			//drawTimeZone();
			var cityLocalTime = App.getApp().getProperty("CityLocalTime");

			// If available, use city returned from web request; otherwise, use raw city from settings.
			// N.B. error response will NOT contain city.
			if ((cityLocalTime != null) && (cityLocalTime["city"] != null)) {
				city = cityLocalTime["city"];
			}

			// Time zone 1 city.
			dc.setColor(gMonoDarkColour, Gfx.COLOR_TRANSPARENT);
			dc.drawText(
				locX + (width / 2),
				mRow1Y,
				gNormalFont,
				// Limit string length.
				city.substring(0, 10),
				Graphics.TEXT_JUSTIFY_CENTER | Graphics.TEXT_JUSTIFY_VCENTER
			);

			// Time zone 1 time.
			var time;
			if (cityLocalTime) {

				// Web request responded with server error e.g. unknown city.
				if (cityLocalTime["error"] != null) {

					time = "???";

				// Web request responded with time zone data for city.
				} else {
					var timeZoneGmtOffset;

					// Use next GMT offset if it's now applicable (new data will be requested shortly).
					if ((cityLocalTime["next"] != null) && (Time.now().value() >= cityLocalTime["next"]["when"])) {
						timeZoneGmtOffset = cityLocalTime["next"]["gmtOffset"];
					} else {
						timeZoneGmtOffset = cityLocalTime["current"]["gmtOffset"];
					}
					timeZoneGmtOffset = new Time.Duration(timeZoneGmtOffset);
					
					var localGmtOffset = Sys.getClockTime().timeZoneOffset;
					localGmtOffset = new Time.Duration(localGmtOffset);

					// (Local time) - (Local GMT offset) + (Time zone GMT offset)
					time = Time.now().subtract(localGmtOffset).add(timeZoneGmtOffset);
					time = Gregorian.info(time, Time.FORMAT_SHORT);
					time = App.getApp().getFormattedTime(time.hour, time.min);
					time = time[:hour] + ":" + time[:min] + time[:amPm]; 
				}

			// Awaiting response to web request sent by BackgroundService.
			} else {
				time = "...";
			}

			dc.setColor(gMonoLightColour, Gfx.COLOR_TRANSPARENT);
			dc.drawText(
				locX + (width / 2),
				mRow2Y,
				gNormalFont,
				time,
				Graphics.TEXT_JUSTIFY_CENTER | Graphics.TEXT_JUSTIFY_VCENTER
			);

		} else {*/
		
		if (mFirstGoalType != -3){ 
			drawGoalValues(dc, mX1,mRow1Y1,mRow2Y1 , mFirstGoalCurrent, mFirstGoalMax, Graphics.TEXT_JUSTIFY_LEFT);
		}
		if (mSecondGoalType != -3){ 
			drawGoalValues(dc, mX2,mRow1Y2,mRow2Y2 , mSecondGoalCurrent, mSecondGoalMax, Graphics.TEXT_JUSTIFY_LEFT);
		}
		//}
	}

	function drawGoalIcon(dc, x, type, isValid, align, goalIconY) {
		 

		var icon = {
			GOAL_TYPE_STEPS => "0",
			GOAL_TYPE_FLOORS_CLIMBED => "1",
			GOAL_TYPE_ACTIVE_MINUTES => "2",
			GOAL_TYPE_BATTERY => "4",
			GOAL_TYPE_CALORIES => "6",
		}[type];

		dc.setColor(isValid ? gThemeColour : gMeterBackgroundColour, Gfx.COLOR_TRANSPARENT);
		dc.drawText(
			x,
			goalIconY,
			gIconsFont,
			icon,
			align
		);
	}

	function drawGoalValues(dc, x, row1y, row2y, currentValue, maxValue, align) {
	/*	var digitStyle = App.getApp().getProperty("GoalMeterDigitsStyle");

		// #107 Only draw values if digit style is not Hidden.
		if (digitStyle != 2 ) {*/
			if (currentValue != null) {
				dc.setColor(gMonoLightColour, Gfx.COLOR_TRANSPARENT);
				dc.drawText(
					x,

					// #107 Draw current value vertically centred if digit style is Current (i.e. not drawing max/target).
					/*(digitStyle == 1 ) ? ((row1y + row2y) / 2) :*/ row1y,

					gNormalFont,
					currentValue,
					align | Graphics.TEXT_JUSTIFY_VCENTER
				);
			}

			// #107 Only draw max/target goal value if digit style is set to Current/Target.
			if ((maxValue != null) /*&& (digitStyle == 0)*/ ) {
				dc.setColor(gMonoDarkColour, Gfx.COLOR_TRANSPARENT);
				dc.drawText(
					x,
					row2y,
					gNormalFont,
					maxValue,
					align | Graphics.TEXT_JUSTIFY_VCENTER
				);
			}
		//}
	}
}
